Module value

Module value 

Structs

PrefixedBlob
PrefixedBlob is a length-prefixed string of bytes.
PrefixedList
A PrefixedList represents a length prefixed list, with a length prefix of L and elements of type T.

Traits

DecoderParameterizedValue
A value whose decoding implementation can be altered by a provided parameter.
DecoderParameterizedValueMut
A mutable value whose decoding implementation can be altered by a provided parameter.
DecoderValue
DecoderValueMut