Struct s2n_codec::unaligned::TryFromIntError
source · pub struct TryFromIntError(_);Trait Implementations
sourceimpl Clone for TryFromIntError
impl Clone for TryFromIntError
sourcefn clone(&self) -> TryFromIntError
fn clone(&self) -> TryFromIntError
Returns a copy of the value. Read more
1.0.0 · sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moresourceimpl Debug for TryFromIntError
impl Debug for TryFromIntError
sourceimpl Display for TryFromIntError
impl Display for TryFromIntError
sourceimpl From<TryFromIntError> for TryFromIntError
impl From<TryFromIntError> for TryFromIntError
sourcefn from(_: TryFromIntError) -> Self
fn from(_: TryFromIntError) -> Self
Converts to this type from the input type.
sourceimpl Hash for TryFromIntError
impl Hash for TryFromIntError
sourceimpl Ord for TryFromIntError
impl Ord for TryFromIntError
sourcefn cmp(&self, other: &TryFromIntError) -> Ordering
fn cmp(&self, other: &TryFromIntError) -> Ordering
1.21.0 · sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere Self: Sized,
Compares and returns the maximum of two values. Read more
sourceimpl PartialEq<TryFromIntError> for TryFromIntError
impl PartialEq<TryFromIntError> for TryFromIntError
sourcefn eq(&self, other: &TryFromIntError) -> bool
fn eq(&self, other: &TryFromIntError) -> bool
This method tests for
self and other values to be equal, and is used
by ==.sourceimpl PartialOrd<TryFromIntError> for TryFromIntError
impl PartialOrd<TryFromIntError> for TryFromIntError
sourcefn partial_cmp(&self, other: &TryFromIntError) -> Option<Ordering>
fn partial_cmp(&self, other: &TryFromIntError) -> Option<Ordering>
1.0.0 · sourcefn le(&self, other: &Rhs) -> bool
fn le(&self, other: &Rhs) -> bool
This method tests less than or equal to (for
self and other) and is used by the <=
operator. Read more