Exclusive

Enum Exclusive 

pub enum Exclusive {}
The Ptr<'a, T> adheres to the aliasing rules of a &'a mut T.

The referent of an exclusively-aliased Ptr may not be concurrently referenced by any other Ptrs or references, and may not be accessed (read or written) other than via this Ptr.