PtrInner

Struct PtrInner 

pub struct PtrInner<'a, T>
where T: ?Sized,
{ /* private fields */ }
The inner pointer stored inside a Ptr.

PtrInner<'a, T> is covariant in 'a and invariant in T.

Implementations

Source

impl<'a, T> PtrInner<'a, T>
where T: 'a + ?Sized,

pub const unsafe fn new(ptr: NonNull<T>) -> PtrInner<'a, T>

Constructs a Ptr from a NonNull.

Safety

The caller promises that:

  1. If ptr’s referent is not zero sized, then ptr has valid provenance for its referent, which is entirely contained in some Rust allocation, A.
  2. If ptr’s referent is not zero sized, A is guaranteed to live for at least 'a.
pub const fn as_non_null(&self) -> NonNull<T>

Converts this PtrInner<T> to a NonNull<T>.

Note that this method does not consume self. The caller should watch out for unsafe code which uses the returned NonNull in a way that violates the safety invariants of self.

pub const fn as_ptr(&self) -> *mut T

Converts this PtrInner<T> to a [*mut T].

Note that this method does not consume self. The caller should watch out for unsafe code which uses the returned *mut T in a way that violates the safety invariants of self.

Source

impl<'a, T> PtrInner<'a, T>
where T: ?Sized,

pub fn from_ref(ptr: &'a T) -> PtrInner<'a, T>

Constructs a PtrInner from a reference.

pub fn from_mut(ptr: &'a mut T) -> PtrInner<'a, T>

Constructs a PtrInner from a mutable reference.

pub fn project<U, C>(self) -> PtrInner<'a, U>
where C: Project<T, U>, U: ?Sized,

Safety

The caller may assume that the resulting PtrInner addresses the subset of the bytes of self’s referent addressed by C::project(self).

Source

impl<'a, T> PtrInner<'a, T>
where T: KnownLayout + ?Sized,

pub fn meta(self) -> MetadataOf<T>

Extracts the metadata of this ptr.

pub unsafe fn with_meta( self, meta: <T as KnownLayout>::PointerMetadata, ) -> PtrInner<'a, T>
where T: KnownLayout,

Produces a PtrInner with the same address and provenance as self but the given meta.

Safety

The caller promises that if self’s referent is not zero sized, then a pointer constructed from its address with the given meta metadata will address a subset of the allocation pointed to by self.

Source

impl<'a, T> PtrInner<'a, T>
where T: KnownLayout<PointerMetadata = usize> + ?Sized,

pub unsafe fn split_at_unchecked( self, l_len: MetadataOf<T>, ) -> (PtrInner<'a, T>, PtrInner<'a, [<T as SplitAt>::Elem]>)
where T: SplitAt,

Splits T in two.

Safety

The caller promises that:

  • l_len.get() <= self.meta().
(Non-)Overlap

Given let (left, right) = ptr.split_at(l_len), it is guaranteed that left and right are contiguous and non-overlapping if l_len.padding_needed_for() == 0. This is true for all [T].

If l_len.padding_needed_for() != 0, then the left pointer will overlap the right pointer to satisfy T’s padding requirements.

pub fn trailing_slice(self) -> PtrInner<'a, [<T as SplitAt>::Elem]>
where T: SplitAt,

Produces the trailing slice of self.

Source

impl<'a, T> PtrInner<'a, [T]>

pub unsafe fn slice_unchecked(self, range: Range<usize>) -> PtrInner<'a, [T]>

Creates a pointer which addresses the given range of self.

Safety

range is a valid range (start <= end) and end <= self.meta().

pub fn iter(&self) -> impl Iterator<Item = PtrInner<'a, T>>

Iteratively projects the elements PtrInner<T> from PtrInner<[T]>.

Source

impl<'a, T, const N: usize> PtrInner<'a, [T; N]>

pub fn as_slice(self) -> PtrInner<'a, [T]>

Casts this pointer-to-array into a slice.

Safety

Callers may assume that the returned PtrInner references the same address and length as self.

Source

impl<'a> PtrInner<'a, [u8]>

pub fn try_cast_into<U>( self, cast_type: CastType, meta: Option<<U as KnownLayout>::PointerMetadata>, ) -> Result<(PtrInner<'a, U>, PtrInner<'a, [u8]>), ConvertError<AlignmentError<PtrInner<'a, [u8]>, U>, SizeError<PtrInner<'a, [u8]>, U>, Infallible>>
where U: 'a + KnownLayout + ?Sized,

Attempts to cast self to a U using the given cast type.

If U is a slice DST and pointer metadata (meta) is provided, then the cast will only succeed if it would produce an object with the given metadata.

Returns None if the resulting U would be invalidly-aligned, if no U can fit in self, or if the provided pointer metadata describes an invalid instance of U. On success, returns a pointer to the largest-possible U which fits in self.

Safety

The caller may assume that this implementation is correct, and may rely on that assumption for the soundness of their code. In particular, the caller may assume that, if try_cast_into returns Some((ptr, remainder)), then ptr and remainder refer to non-overlapping byte ranges within self, and that ptr and remainder entirely cover self. Finally:

  • If this is a prefix cast, ptr has the same address as self.
  • If this is a suffix cast, remainder has the same address as self.