pub struct PrefixedList<'a, L, T> {
pub list: &'a [T],
/* private fields */
}Expand description
A PrefixedList represents a length prefixed list, with a length prefix of L
and elements of type T.
Note that this will neither allocate nor copy T, so it must be valid to directly
construct them from the underlying &[u8].
This type is particularly useful for representing TLS messages, such as a list
of supported NamedGroup items in the Supported Groups extension.
Fields
list: &'a [T]Trait Implementations
Sourceimpl<'a, L: Into<usize> + DecoderValue<'a>, T: FromBytes + Immutable + Unaligned> DecoderValue<'a> for PrefixedList<'a, L, T>
impl<'a, L: Into<usize> + DecoderValue<'a>, T: FromBytes + Immutable + Unaligned> DecoderValue<'a> for PrefixedList<'a, L, T>
fn decode(bytes: DecoderBuffer<'a>) -> DecoderBufferResult<'a, Self>