Module zerocopy
Source - byteorder
- Byte order-aware numeric primitives.
- include_value
- Includes a file and safely transmutes it to a value of an arbitrary type.
- transmute
- Safely transmutes a value of one type to a value of another type of the same
size.
- transmute_mut
- Safely transmutes a mutable reference of one type to an mutable reference of
another type of the same size.
- transmute_ref
- Safely transmutes a mutable or immutable reference of one type to an
immutable reference of another type of the same size.
- F32
- A 32-bit floating point number stored in a given byte order.
- F64
- A 64-bit floating point number stored in a given byte order.
- I16
- I32
- I64
- I128
- Ptr
- A raw pointer with more restrictions.
- Ref
- A typed reference derived from a byte slice.
- U16
- U32
- U64
- U128
- Unalign
- A type with no alignment requirement.
- BigEndian
- Defines big-endian serialization.
- LittleEndian
- Defines little-endian serialization.
- AsBytes
- Types that can be viewed as an immutable slice of initialized bytes.
- ByteOrder
ByteOrder describes types that can serialize integers as bytes.- ByteSlice
- A mutable or immutable reference to a byte slice.
- ByteSliceMut
- A mutable reference to a byte slice.
- FromBytes
- Types for which any bit pattern is valid.
- FromZeroes
- Types for which a sequence of bytes all set to zero represents a valid
instance of the type.
- Unaligned
- Types with no alignment requirement.
- BE
- A type alias for
BigEndian. - LE
- A type alias for
LittleEndian. - NativeEndian
- Defines system native-endian serialization.
- NetworkEndian
- Defines network byte order serialization.
- AsBytes
- FromBytes
- FromZeroes
- KnownLayout
- Unaligned