Modules
- Byte order-aware numeric primitives.
Macros
- Includes a file and safely transmutes it to a value of an arbitrary type.
- Safely transmutes a value of one type to a value of another type of the same size.
- Safely transmutes a mutable reference of one type to an mutable reference of another type of the same size.
- Safely transmutes a mutable or immutable reference of one type to an immutable reference of another type of the same size.
Structs
- A 32-bit floating point number stored in a given byte order.
- A 64-bit floating point number stored in a given byte order.
- A raw pointer with more restrictions.
- A typed reference derived from a byte slice.
- A type with no alignment requirement.
Enums
- Defines big-endian serialization.
- Defines little-endian serialization.
Traits
- Types that can be viewed as an immutable slice of initialized bytes.
ByteOrderdescribes types that can serialize integers as bytes.- A mutable or immutable reference to a byte slice.
- A mutable reference to a byte slice.
- Types for which any bit pattern is valid.
- Types for which a sequence of bytes all set to zero represents a valid instance of the type.
- Types with no alignment requirement.
Type Aliases
- A type alias for
BigEndian. - A type alias for
LittleEndian. - Defines system native-endian serialization.
- Defines network byte order serialization.