Module s2n_codec::zerocopy

source ·

Modules

  • Byte order-aware numeric primitives.

Macros

  • Safely transmutes a value of one type to a value of another type of the same size.

Structs

Enums

Traits

  • Types which are safe to treat as an immutable byte slice.
  • ByteOrder describes types that can serialize integers as bytes.
  • A mutable or immutable reference to a byte slice.
  • A mutable reference to a byte slice.
  • Types for which any byte pattern is valid.
  • Types with no alignment requirement.

Functions

  • Reinterprets the bits of a value of one type as another type.

Type Definitions

Derive Macros