Modules
- Byte order-aware numeric primitives.
Macros
- Safely transmutes a value of one type to a value of another type of the same size.
Structs
- A length- and alignment-checked reference to a byte slice which can safely be reinterpreted as another type.
- A type with no alignment requirement.
Enums
- Defines big-endian serialization.
- Defines little-endian serialization.
Traits
- Types which are safe to treat as an immutable byte slice.
ByteOrderdescribes types that can serialize integers as bytes.- A mutable or immutable reference to a byte slice.
- A mutable reference to a byte slice.
- Types for which any byte pattern is valid.
- Types with no alignment requirement.
Functions
- Reinterprets the bits of a value of one type as another type.
Type Definitions
- A type alias for
BigEndian. - A type alias for
LittleEndian. - Defines system native-endian serialization.
- Defines network byte order serialization.