pub struct Ptr<'a, T>where
T: 'a + ?Sized,{ /* private fields */ }Expand description
A raw pointer with more restrictions.
Ptr<T> is similar to NonNull<T>, but it is more restrictive in the
following ways:
- It must derive from a valid allocation
- It must reference a byte range which is contained inside the
allocation from which it derives
- As a consequence, the byte range it references must have a size
which does not overflow
isize
- As a consequence, the byte range it references must have a size
which does not overflow
- It must satisfy
T’s alignment requirement
Thanks to these restrictions, it is easier to prove the soundness of
some operations using Ptrs.
Ptr<'a, T> is covariant in 'a and T.